/* *****************************************************************
 * High Speed Serial Driver for the Dynamic Engineering PMC BiSerial
 * This driver only ddoes transmit... ie implement write  system call
 * This driver is Copyright David Armstrong 2000, 2001
 * This source code is released under GPL see the file gpl.txt for 
 * details
 *******************************************************************
 */ 

 #define HSS_IOC_MAGIC 'H'

 /* Refer to LINUX DEVICE DRIVERS Chapter 5 */
 /* MACROS to be used with IOCTL system calls to controll the hss device */


 #define BiRESET             _IO(HSS_IOC_MAGIC,0)
 #define BiTX_FIFO_SET_AE    _IOW(HSS_IOC_MAGIC,9,16)
 #define BiRX_FIFO_SET_AF    _IOW(HSS_IOC_MAGIC,10,16)
 #define BiTX_FIFO_WRITE     _IOW(HSS_IOC_MAGIC,11,16)
 #define BiTX_FIFO_READ	     _IOR(HSS_IOC_MAGIC,12,16)
 #define BiRX_FIFO_WRITE     _IOW(HSS_IOC_MAGIC,13,16)
 #define BiRX_FIFO_READ	     _IOR(HSS_IOC_MAGIC,14,16)
 #define BiSTAT0_READ        _IOR(HSS_IOC_MAGIC,15,32)
 #define BiRx_READ           _IOR(HSS_IOC_MAGIC,16,32)
 #define BiRx_WRITE          _IOW(HSS_IOC_MAGIC,17,32)
 #define BiBASE_READ         _IOR(HSS_IOC_MAGIC,18,32)
 #define BiBASE_WRITE        _IOW(HSS_IOC_MAGIC,19,32)
 #define BiDIR_TERM_WRITE    _IOW(HSS_IOC_MAGIC,20,32)
 #define BiTx_READ           _IOR(HSS_IOC_MAGIC,21,32)
 #define BiTx_WRITE          _IOW(HSS_IOC_MAGIC,22,32)
 #define BiTx_WAIT           _IOR(HSS_IOC_MAGIC,23,32)


/* PMC register bit DefinitionsT*/

/* bis Base */

#define FRX_LD   (1<<22)
#define FTX_LD   (1<<21)
#define FIFO_EN  (1<<20)
#define INT_SET (1<<17)
#define INT_EN_MASTER (1<<16)
#define CLOCK_OSC (0)
#define CLOCK_EXT (1<<14)
#define CLOCK_PCI (3<<13)
#define CLOCK_DIV  (1<<12)

/* bis tx */
#define CLOCK_EN (1<<3)    /* 1 = drive TX clock */
#define INT_EN_PAE (1<<2)  /* Almost empty interrupt enable*/
#define INT_EN_TX (1<<1)   /* TX Interrupt enable */
#define STRT_TX  0x0001    /* start transmit */
/* bis rx */
#define TESTMODE (1<<3)    /* 1 = use PCI clock for RX clock */
#define INT_EN_PAF (1<<2)  /* Almost full interrupt enable*/
#define INT_EN_RX (1<<1)   /* RX Interrupt enable */
#define STRT_RX  0x0001    /* start transmit */

/* bis stat0 */

#define INT_REQUEST (1<<15)    /* Interrupt request */
#define FAE_INT     (1<<12)    /* TX almost empty Interrupt */
#define TX_INT      (1<<11)    /* TX interrupt */
#define FTX_FFN     (1<<10)    /* TX FIFO full = 0 */
#define FTX_FAEN    (1<<9)     /* TX FIFO almost empty = 0 */
#define FTX_AEO FTX_FAEN       /* so we can use the same as IP */
#define FTX_MTN     (1<<8)     /* TX FIFO empty = 0 */
#define FTX_EMPTY FTX_MTN      /* so we can use the same code as IP version */

#define FAF_INT     (1<<4)     /* RX almost empty Interrupt */
#define RX_INT      (1<<3)     /* RX interrupt */
#define FRX_FFN     (1<<2)     /* RX FIFO full = 0 */
#define FRX_FULL     FRX_FFN   /* so we can use the same code as IP version */
#define FRX_FAFN    (1<<1)     /* RX FIFO almost full = 0 */
#define FRX_AFO      FRX_FAFN  /* so we can use the same code as IP version */

#define FRX_MTN     (1<<0)     /* RX FIFO empty = 0 */


/* There are 20 IO lines                      */
/* IO 0..3 are individually controllable      */
/* the remainder are controllable in groups 4 */
/* This takes 8 bits                          */
/* 0,12,13,14 are inputs                      */
/* 4,5,6    are outputs                       */
/* bits    IO                                 */
/*    7    16 - 19                            */
/*    6    12 - 15 inputs                     */
/*    5     8 - 11                            */
/*    4     4 - 7  outputs                    */
/*    3     3                                 */
/*    2     2                                 */
/*    1     1                                 */
/*    0     0      input                      */
/* inputs should probably be terminated, same setup shifted left 16 bits */


#define DEFAULT_DRIVE 0x10
#define DEFAULT_TERM  ((0x41) <<16)  // terminate rx clock
#define DEF_DRIVE_TERM DEFAULT_DRIVE + DEFAULT_TERM

