
/* *****************************************************************
 * High Speed Serial Driver for the Dynamic Engineering PMC BiSerial
 * This driver only ddoes transmit... ie implement write  system call
 * This driver is Copyright David Armstrong 2000, 2001
 * This source code is released under GPL see the file gpl.txt for 
 * details
 *******************************************************************
 * This module generates random data streams using the selected polynomial
 *
 */ 


const long int Poly[] =
        {   0x1,       // 1
            0x3,       // 2
            0x5,       // 3
            0x3,       // 4
            0x5,       // 5
            0x3,       // 6
            0x3,       // 7
            0x63,      // 8
            0x11,      // 9 bit polynomial worldspace 9th order
            0x09,      // 10
            0x05,      // 11
            0x99,      // 12
            0x1b,      // 13
            0x1803,    // 14
            0x03,      // 15
            0x2d       // 16
         };

static long int InitialValue = -1L;
static long int ShiftReg;

static int PolyDegree = 9;  /* feed_back tap location*/
static long int PolyFeedBack;

void InitRandom(int PolyDegree )
{
  PolyFeedBack = 1<< (PolyDegree-1);
  InitialValue = (1<< PolyDegree) -1;
  ShiftReg = InitialValue; 
}


unsigned char RandomByte(void)
{

  static long int temp;
  int bit;
  int FeedBack;
  int ScrByte;
  
  /* set up the initial condition */

  for (bit = 0; bit < 8 ; bit ++)
  {
    /* The high order bit of a byte is transmitted first.*/
    /* This then requires that it be scrambled with the first bit out of */
    /* the scrambler.  So rather than reverse the scrambler we reverse  */
    /* the bit order of in the byte.*/
    ScrByte <<= 1;
    ScrByte |= (ShiftReg & 0x1);
    temp = ShiftReg & Poly[PolyDegree-1];
    FeedBack = 0;
    do
    {
      FeedBack = FeedBack ^ (temp & 0x1);
      temp >>= 1;
    } while(temp != 0);

    ShiftReg = ShiftReg >> 1;      
    if (FeedBack)
      ShiftReg |= PolyFeedBack;
  }
  return ScrByte & 0xff;
}


