
/* *****************************************************************
 * High Speed Serial Driver for the Dynamic Engineering PMC BiSerial
 * This driver only ddoes transmit... ie implement write  system call
 * This driver is Copyright David Armstrong 2000, 2001
 * This source code is released under GPL see the file gpl.txt for 
 * details
 *******************************************************************
 This program is a test program, it is able to exercise the card with 
 real or random data.  Us it as a starting point for a real application

 */ 


#include <sys/types.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <syscall.h>
#include <sys/timeb.h>
#include <math.h>
#include <sys/ioctl.h>
#include <stream.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include "hss.h"


enum clock_source {Internal,External,Pci} source;
enum data_type  {zero,one,arb,Random,file,alphabet} optData;

extern unsigned char RandomByte(void);
extern void InitRandom(int degree);
void usage(void)
{
  cout <<"\n\ntest_datan";
  cout <<"\n Command line options:";
  cout <<"\n -? Displays these usage notes.";
  cout <<"\n -A enables alphabet generation.";
  cout <<"\n -Bnnn changes the default (8000) fifo empty threshold.";
  cout <<"\n -DR enables Random data generation.";
  cout <<"\n -D0 causes payload data to be all zeros.";
  cout <<"\n -D1 causes payload data to be all ones.";
  cout <<"\n -DXnn sets payload data bytes to nn (nn is hex value).";
  cout <<"\n -E enables use of external data clock (DEFAULT)";
  cout <<"\n -F writes to file -- file_dump in the current directory";
  cout <<"\n -H enables hex dump mode: builds and dumps frame buffer then exits.";
  cout <<"\n -IXXX enables use of internal data clock with divisor XXX";
  cout <<"\n -Nn changes the number of bytes sent from (default - 96000)";
  cout <<"\n -PXXX enable use of PCI clock with divisor XXX";
  cout <<"\n -Tfilename transfers a data file to the output.";
}

unsigned char AlphaByte(void)
{
    static int bitshift =0;
    static int temp;
    int temp2;
    static int letter = 'Z';
    letter = letter ++;
    if (letter > 'Z')
    {
        bitshift++;
        if (bitshift == 8) bitshift = 0;
        letter = 'A';
    }
    temp = (temp & 0xff00) | letter;
    temp2 = (temp >> bitshift) & 0xff;
    temp = temp << 8;
    
    return (unsigned char) temp2;
}


void dump_buffer( unsigned char buff[],unsigned long len)
{
  unsigned int i,j;

  printf("\n\nFrame Buffer Contents:\n");
  printf("Frame buffer length = %lud (%lx) bytes [%lud bits]\n", len, len, len*8);
  for (i=0; i<len; i = i + 16)
  {
    if ((i>0) && ((i % 0x1000) == 0))
      printf("\n");
    printf("%04X: ", i);
    for (j=i; (j<len)&&(j<i+16); j++)
    {
      if (j % 2 != 0)
        printf(":");
      else
        printf(" ");
      printf("%02X",buff[j]);
    }
    printf("\n");
  }
  cout << "\n i = " << i;
  printf(" \nEnd of Buffer\n");
}

void write_datablock(unsigned char c[],unsigned int idx, unsigned char data, unsigned int length)
{
    unsigned int j;

    for (j = 0 ; j < length; j++)
    {
       c[idx^3] = data;
       idx++;
    }

}

//
// atox - ASCII character to hexadecimal value conversion
//
unsigned char atox(unsigned char c)
{
    unsigned char x;

    x = toupper(c);
    if ( (x >= '0') && (x <= '9') )
        x -= '0';
    else if ( (x >= 'A') && (x <= 'F') )
        x = (x - 'A') + 10;
    return(x);
}


long fsize(char *fname)
{

	FILE *stream;
	unsigned char c;
	long byte_count;

	if ((stream = fopen(fname, "rb")) == NULL)
	{
		return(-1);
	}
	for (byte_count=0; !feof(stream); byte_count++)
	{
		c = getc(stream);
	}
	fclose(stream);
	return(byte_count-1);
}


long FillBufferWithFile(char *fname, unsigned char **buf)
{
   FILE *stream;
   long write_idx;
   long FileSize;
   long BufferSize;

   FileSize = fsize(fname);

   if (FileSize < 0)
   {
	   cout << "\n\nCan't open file: '" << fname << "'.\n";
	   return(-1);
   }
   else if (FileSize == 0)
   {
	   cout << "\n\nThe file '" << fname << "' contains no data!\n";
	   return(-1);
   }

   cout << "\n\n file '" << fname << "' contains " << FileSize << " octets.\n";

   BufferSize = FileSize;

   *buf = (unsigned char *)malloc(BufferSize+32);
   if (*buf == (unsigned char *)0)
   {
		cout << "\n\nCan't allocate required memory: " << BufferSize+32 << " bytes.";
		return(-1);
   }

   memset(*buf,0x00, BufferSize);
   
   write_idx = 0;
   if ((stream = fopen(fname, "rb")) == NULL)
   {
      cout << "\n\nCan't open file: '" << fname << "'.\n";
   }
   else
   {
       while (!feof(stream))
	   {
          (*buf)[write_idx^3] = fgetc(stream);
		  write_idx++;
	   }
   }
   cout << "buffer fill is complete!\n";
   return (write_idx-1);
}



int main ( int argc , char ** argv )
{
  const float Oscillator = 10000000.0;
  const float PciClock = 33000000.0;
  
  long h,i, write_count ;
  long result,total_count;
  long offset;

  clock_source source;  // Command line option -I : enable use of internal clock
                        //                     -E : external clock
                        //                     -P : PCI clock
                        //                     default external
  int divisor = 128;    // default divisor
  bool optDump;       // Command line option -D : dumps frame data
  bool optWrite;      // Command line option -F : writes to a file
  bool optRandomData; // Command line option -DR : randomizes data blocks
                      // default value : false
  bool optDataSet;    // Command line option -D0, -D1, or -Dxnn specifies a
                      //  specific value for payload data.

  int empty_threshold = 8000;


  long int NuBytes = 960000 ; // number of bytes to send
  // 960000 bytes takes 60 seconds to send at 128kbps
  char *data_filename;

  char filename[255];
  FILE* stream;

  union {
	  unsigned char *buff;
	  long *long_buff;
  } b;

  unsigned char arbdata;


  cout <<"\nTest program for Dynamic Engineering BiSerial PMC";
  cout <<"\n  Dec 12, 2000";

  optDump = false;
  optWrite = false;
  source = Pci;
  optRandomData = false;
  optDataSet = false;
  arbdata = 0;
  data_filename = (char *)0;

  if (argc > 1)
  {
      for (i=1; i<argc; i++)
      {
          if (argv[i][0] == '-')
	      {
	          switch (toupper(argv[i][1]))
	          {
                  case 'A':
                      optData = alphabet;
                      break;

                  case 'B':            // -Bnnn changes empty threshold 
                      empty_threshold = atoi(&argv[i][2]);
                      cout << "Empty Threshold = " << empty_threshold << endl; 
                      break;
                  case 'D':            // -Dx sets type of payload data
                      switch (toupper(argv[i][2]))
                      {
                          case 'R':
                              optData = Random;
                              break;

                          case 'X':
                              arbdata = atoi(&argv[i][3]);
                              optData = arb;
                              break;

                          case '1':
                              arbdata = 0xff;
                              optData = arb;
                              break;

                          case '0':
                              arbdata = 0x00;
                              optData = zero;
                          default:
                              break;
                       }
                       break;

                  case 'E':            // -E enables use of external clock
                      source = External;
                      break;

                  case 'F':            // -F writes to a file
                      optWrite = true;
                      break;

                  case 'H':            // -H enables hex dump mode
                      optDump = true;
                      break;

                  case 'I':            // -I enables use of internal 10 MHz clock
                      source = Internal;
                      break;

                  case 'N':            //  -N sets the number of bytes to send
                      NuBytes = atoi(&argv[i][2]);
                      break;

                  case 'P':            // -I enables use of internal clock
                      source = Pci;
                      break;

				  case 'T':            // -T reads a data file into the buffer
                      data_filename = &argv[i][2];
					  break;

                  case '?':            // -? displays usage notes
                      usage();
                      exit(1);
                      break;

                  default:
                      usage();
                      exit(1);
                      break;
              }
          }
      }
  }
  long BufferSize = NuBytes+32;
  if (data_filename != (char *)0)
  { 
      cout <<endl << endl <<"Reading data file: '" << data_filename << "'." <<  endl ;
      BufferSize = FillBufferWithFile(data_filename, &(b.buff));
      NuBytes= BufferSize;
  }
  else
  {
     b.buff = (unsigned char *)malloc(BufferSize+32);
     if (b.buff == (unsigned char *)0)
     {
		cout << "\n\nCan't allocate required memory: " << BufferSize+32 << " bytes.";
		return(-1);
     }
     if (optData == Random)
     {
         InitRandom(10);
         cout << "Filling buffer with Random Data " << endl;
         for (i=0 ; i < BufferSize ; i++)
         {
             b.buff[i] = RandomByte();
         }
     }
     else if (optData == alphabet)
     {
         cout << "Filling buffer with alphabet, shifted by 1 bit each interation " << endl;
         
         for (i=0 ; i < BufferSize ; i++)
         {
             b.buff[i] = AlphaByte();
         }
     }
     else
     {
         cout << "Filling buffer with " << hex(arbdata) <<  endl;
         memset(b.buff,arbdata, BufferSize);
     }
  }
  cout << "\nBuffer size is " << BufferSize << ".\n";
  
  if (optDump)
  {
     dump_buffer(b.buff,BufferSize);
     free(b.buff);
     return(1);
  }

  char Name[1024];
  sprintf ( Name , "/dev/hss0");
  h = open ( Name , O_WRONLY );
  if ( h < 2 )
  {
      cout << Name <<"\nFAILED open"<< endl;
      return 1;
  }
  else cout << "\n /dev/hss0 Open successful";

  ioctl(h,BiRESET,&result);
  cout <<"\nPlacing in test mode";
  ioctl(h,BiRx_READ,&result);
  result = result | TESTMODE;;
  ioctl(h,BiRx_WRITE,&result);
  cout <<"\nWriting Rx with: " << hex(result);

  offset = empty_threshold;  // larger than one byte
  cout <<"\n  Reseting TX FIFO set points with " << offset << endl;
  ioctl(h,BiTX_FIFO_SET_AE,&offset );
  // WARNING DO NOT program the rx offset after the tx offset as it will reset
  // the tx !!!!!  This is a deficency of the driver and could be corrected but
  // is not necessary for its required purpose DBA 2000,09,18
  // 
  //ioctl(h,BiBASE_READ,&result);  /* this is here to make the RX FIFO setting work?? */
  //ioctl(h,BiRX_FIFO_SET_AF,&offset );


  // set up baud rate
  //use internal clock
  float clock;
  float reference;

  ioctl(h,BiBASE_READ,&result);

  result = result & (~(const long int)CLOCK_PCI); /* force the clock select bits to zero */

  switch (source)
	{
     case Internal:
          reference = Oscillator;
          result |= CLOCK_OSC;
          break;

     case External:
          
          result |= CLOCK_EXT;
          break;

     case Pci:
          reference = PciClock;
          result |= CLOCK_PCI;
          break;
  }
  if (source == External)
  {
     clock = reference;
  }
  else
  {
      result = result | CLOCK_DIV;
      result = result | divisor;
      clock = reference/(2*(divisor+1));
  }
  ioctl(h,BiBASE_WRITE,&result);
  cout << "\nWriting clock reg with: " << hex(result);
  cout << "\nWriting drive termination register with "<< hex(DEF_DRIVE_TERM);
  ioctl(h,BiDIR_TERM_WRITE, DEF_DRIVE_TERM);

  if (source!= External)
  {
      cout <<"\nSetting up internal clock:  "<< clock  <<" bps \n";
  }
  else
  {
      cout <<"\nSetting up external clock \n";
  }
  float seconds = 0;
  long milliseconds = 0;
  float bits_per_second = 0;
  struct timeb StartTime;
  struct timeb EndTime;


  // TEST the speed of the interface
  cout <<"\n\nStarting Test";
  cout <<"\n Test program will generate ";
  cout << NuBytes << " bytes " << endl;

  if (optData == Random )
  {
      cout <<" with random data"<< endl;
  }
  else if (data_filename != (char *)0)
  {
      cout <<"read from data file: '" << data_filename << endl ;
  }
  else if (arbdata == 0)
  {
      cout << " 00  data" << endl;
  }
  else
  {
      cout << hex( arbdata) << " data" << endl;
  }

  total_count = 0;
  i = 0;
  long read_idx = 0;
  ftime(&StartTime);
  long WriteBlockLength = 1600;
  

  if( optWrite )
  {
      // open output file
      strcpy( filename,"written_data_file" );
      stream = fopen( filename, "w" );
  }
  
  do
  {
        if (optWrite)
        {
            write_count = fwrite( &b.buff[read_idx], sizeof(char), WriteBlockLength, stream);
        }
        else
        {
            write_count = write(h,&b.buff[read_idx],WriteBlockLength );
        }
        total_count += write_count;
        read_idx += write_count;
        if ((read_idx + WriteBlockLength) >= BufferSize)
        {
              break;
        }
  }
  while (write_count > 0) ;
  if (optWrite)
  {
      fclose(stream);
      exit(0);
  }
  else
  {
      // make blocking call to ioctl that blocks until ints are off
      cout << "Sleeping until complete " << endl;  
      ioctl(h, BiTx_WAIT, &result);
  }
  ftime(&EndTime);



  milliseconds = EndTime.millitm -StartTime.millitm;
  if (milliseconds < 0)
  {
     milliseconds += 1000;
     EndTime.time -=1;
  }

  free(b.buff);
  seconds =(float) ((EndTime.time - StartTime.time)*1000 + milliseconds)/1000.0;
  bits_per_second = (float) (total_count * 8)/seconds;
  cout <<"\n  Wrote " << total_count << " bytes in " << seconds << " seconds " << bits_per_second << " bits/second\n ";

  close(h);
  return 0;
 
}
